@@ -9,13 +9,13 @@ api_mistral <- new_class("Mistral", api_openai)
 #' Extract rate limit info from  Mistral API-Headers
 #'
 #' @noRd
-method(ratelimit_from_header, list(api_mistral,new_S3_class("httr2_headers"))) <- function(api,headers){
-  request_time <- strptime(headers[["date"]], format = "%a, %d %b %Y %H:%M:%S", tz = "GMT")
+method(ratelimit_from_header, list(api_mistral,new_S3_class("httr2_headers"))) <- function(.api,.headers){
+  request_time <- strptime(.headers[["date"]], format = "%a, %d %b %Y %H:%M:%S", tz = "GMT")
   
   # Extract tokens remaining and reset time
-  ratelimit_tokens_remaining <- as.integer(headers[["x-ratelimitbysize-remaining-minute"]])
-  ratelimit_tokens_limit <- as.integer(headers[["x-ratelimitbysize-limit-minute"]])
-  ratelimit_tokens_reset_dt <- as.integer(headers[["ratelimitbysize-reset"]])
+  ratelimit_tokens_remaining <- as.integer(.headers[["x-ratelimitbysize-remaining-minute"]])
+  ratelimit_tokens_limit <- as.integer(.headers[["x-ratelimitbysize-limit-minute"]])
+  ratelimit_tokens_reset_dt <- as.integer(.headers[["ratelimitbysize-reset"]])
   ratelimit_tokens_reset_time <- request_time + ratelimit_tokens_reset_dt
   
   # For requests per second limit
@@ -31,21 +31,120 @@ method(ratelimit_from_header, list(api_mistral,new_S3_class("httr2_headers"))) <
   )
 }
 
+#' Prepare Mistral API request parameters
+#'
+#' @description
+#' Helper function to prepare common request parameters for Mistral API calls,
+#' centralizing duplicated logic between chat and batch operations
+#'
+#' @param .llm An LLMMessage object containing conversation history
+#' @param .model The model identifier to use
+#' @param .max_tokens Maximum tokens for completion
+#' @param .frequency_penalty Frequency penalty parameter
+#' @param .logit_bias Logit bias parameters
+#' @param .presence_penalty Presence penalty parameter
+#' @param .seed Random seed for reproducibility
+#' @param .stop Sequence to stop generation
+#' @param .temperature Temperature parameter
+#' @param .top_p Top p parameter
+#' @param .json_schema JSON schema for structured output
+#'
+#' @return A list containing prepared request parameters
+#' @noRd
+prepare_mistral_request <- function(
+    .llm,
+    .api,
+    .model = "mistral-large-latest",
+    .min_tokens = NULL,
+    .max_tokens = NULL,
+    .frequency_penalty = NULL,
+    .logit_bias = NULL,
+    .presence_penalty = NULL,
+    .seed = NULL,
+    .stop = NULL,
+    .safe_prompt =NULL,
+    .temperature = NULL,
+    .top_p = NULL,
+    .json_schema = NULL
+) {
+  # Format messages using the API format
+  messages <- to_api_format(.llm, .api, TRUE)
+  
+  # Handle JSON schema and response format
+  response_format <- NULL
+  json <- FALSE
+  if (!is.null(.json_schema)) {
+    json <- TRUE
+    schema_name <- "empty"
+    if (requireNamespace("ellmer", quietly = TRUE)) {
+      # Handle ellmer json schema objects
+      if (S7_inherits(.json_schema, ellmer::TypeObject)) {
+        .json_schema <- to_schema(.json_schema)
+        schema_name <- "ellmer_schema"
+      }
+    }
+    if (schema_name != "ellmer_schema") {
+      schema_name <- attr(.json_schema, "name")
+    }
+    
+  
+    # Ensure additionalProperties is set to FALSE if not already provided
+    if (is.null(.json_schema$additionalProperties)) {
+      .json_schema$additionalProperties <- FALSE
+    }
+    
+    response_format <- list(
+      type = "json_schema",
+      json_schema = list(
+        schema = .json_schema,
+        name = schema_name,
+        strict = TRUE  # Mistral API expects this field
+      )
+    )
+  }
+  
+  # Build common request body parameters
+  request_body <- list(
+    model = .model,
+    messages = messages,
+    frequency_penalty = .frequency_penalty,
+    logit_bias = .logit_bias,
+    max_tokens = .max_tokens,  # Note: This might also need to be max_tokens not max_completion_tokens
+    presence_penalty = .presence_penalty,
+    response_format = response_format,
+    random_seed = .seed,  # Note: Mistral uses random_seed not seed
+    safe_prompt = .safe_prompt,
+    stop = .stop,
+    temperature = .temperature,
+    top_p = .top_p
+  ) |> purrr::compact()
+  
+  # Return all prepared elements
+  list(
+    messages = messages,
+    request_body = request_body,
+    json = json,
+    no_system_prompt = TRUE
+  )
+}
 
 #' Send LLMMessage to Mistral API
 #'
 #' @param .llm An `LLMMessage` object.
 #' @param .model The model identifier to use (default: `"mistral-large-latest"`). 
-#' @param .stream Whether to stream back partial progress to the console. (default: `FALSE`).
-#' @param .temperature Sampling temperature to use, between `0.0` and `1.5`. Higher values make the output more random, while lower values make it more focused and deterministic (default: `0.7`).
-#' @param .top_p Nucleus sampling parameter, between `0.0` and `1.0`. The model considers tokens with top_p probability mass (default: `1`).
-#' @param .max_tokens The maximum number of tokens to generate in the completion. Must be `>= 0` (default: `1024`).
+#' @param .frequency_penalty Number between -2.0 and 2.0. Positive values penalize new tokens based on their existing frequency.
+#' @param .logit_bias A named list modifying the likelihood of specified tokens appearing in the completion.
+#' @param .presence_penalty Number between -2.0 and 2.0. Positive values penalize new tokens based on whether they appear in the text so far.
+#' @param .seed If specified, the system will make a best effort to sample deterministically.
+#' @param .stop Up to 4 sequences where the API will stop generating further tokens.
+#' @param .stream If set to TRUE, the answer will be streamed to console as it comes (default: FALSE).
+#' @param .temperature What sampling temperature to use, between 0 and 2. Higher values make the output more random.
+#' @param .top_p An alternative to sampling with temperature, called nucleus sampling.
 #' @param .min_tokens The minimum number of tokens to generate in the completion. Must be `>= 0` (optional).
+#' @param .max_tokens An upper bound for the number of tokens that can be generated for a completion.
 #' @param .tools Either a single TOOL object or a list of TOOL objects representing the available functions for tool calls.
 #' @param .tool_choice A character string specifying the tool-calling behavior; valid values are "none", "auto", or "required".
-#' @param .seed The seed to use for random sampling. If set, different calls will generate deterministic results (optional).
-#' @param .stop Stop generation if this token is detected, or if one of these tokens is detected when providing a list (optional).
-#' @param .json Whether the output should be in JSON mode(default: `FALSE`).
+#' @param .json_schema A JSON schema object provided by tidyllm schema or ellmer schemata.
 #' @param .safe_prompt Whether to inject a safety prompt before all conversations (default: `FALSE`).
 #' @param .timeout When should our connection time out in seconds (default: `120`).
 #' @param .verbose Should additional information be shown after the API call? (default: `FALSE`)
@@ -54,23 +153,26 @@ method(ratelimit_from_header, list(api_mistral,new_S3_class("httr2_headers"))) <
 #' @return Returns an updated `LLMMessage` object.
 #' @export
 mistral_chat <- function(.llm,
-                    .model = "mistral-large-latest",
-                    .stream = FALSE,
-                    .seed = NULL,
-                    .json = FALSE,
-                    .temperature = 0.7,
-                    .top_p = 1,
-                    .stop = NULL,
-                    .safe_prompt = FALSE,
-                    .timeout = 120,
-                    .max_tries = 3,
-                    .max_tokens = 1024,
-                    .min_tokens = NULL,
-                    .dry_run = FALSE,
-                    .verbose = FALSE,
-                    .tools = NULL,
-                    .tool_choice = NULL) {
-
+                         .model = "mistral-large-latest",
+                         .frequency_penalty = NULL,
+                         .logit_bias = NULL,
+                         .presence_penalty = NULL,
+                         .seed = NULL,
+                         .stop = NULL,
+                         .stream = FALSE,
+                         .temperature = 0.7,
+                         .top_p = 1,
+                         .min_tokens = NULL,
+                         .max_tokens = NULL,
+                         .json_schema = NULL,
+                         .safe_prompt = FALSE,
+                         .timeout = 120,
+                         .max_tries = 3,
+                         .dry_run = FALSE,
+                         .verbose = FALSE,
+                         .tools = NULL,
+                         .tool_choice = NULL) {
+  
   # Validate the inputs
   c(
     "Input .llm must be an LLMMessage object" = S7_inherits(.llm, LLMMessage),
@@ -78,14 +180,17 @@ mistral_chat <- function(.llm,
     "Input .stream must be logical" = is.logical(.stream) && length(.stream) == 1,
     "Input .temperature must be numeric between 0 and 1.5" = is.null(.temperature) || (is.numeric(.temperature) && .temperature >= 0 && .temperature <= 1.5),
     "Input .top_p must be numeric between 0 and 1" = is.null(.top_p) || (is.numeric(.top_p) && .top_p >= 0 && .top_p <= 1),
-    "Input .max_tokens must be integer >= 0" = is.null(.max_tokens) || (is_integer_valued(.max_tokens) && .max_tokens >= 0),
+    "Input .max_tokens must be NULL or a positive integer" = is.null(.max_tokens) || (is_integer_valued(.max_tokens) && .max_tokens > 0),
     "Input .min_tokens must be integer >= 0 if provided" = is.null(.min_tokens) || (is_integer_valued(.min_tokens) && .min_tokens >= 0),
     "Input .seed must be integer >= 0 if provided" = is.null(.seed) || (is_integer_valued(.seed) && .seed >= 0),
     "Input .stop must be a string or vector of strings if provided" = is.null(.stop) || (is.character(.stop)),
-    "Input .json must be logical" = is.logical(.json) && length(.json) == 1,
+    "Input .frequency_penalty must be numeric or NULL" = is.null(.frequency_penalty) || is.numeric(.frequency_penalty),
+    "Input .logit_bias must be a list or NULL" = is.null(.logit_bias) || is.list(.logit_bias),
+    "Input .presence_penalty must be numeric or NULL" = is.null(.presence_penalty) || is.numeric(.presence_penalty),
     "Input .safe_prompt must be logical" = is.logical(.safe_prompt) && length(.safe_prompt) == 1,
     "Input .max_tries must be integer-valued numeric" = is_integer_valued(.max_tries),
     "Input .timeout must be integer-valued numeric (seconds till timeout)" = is_integer_valued(.timeout),
+    "Input .json_schema must be NULL or a list or an ellmer type object" = is.null(.json_schema) || is.list(.json_schema) || is_ellmer_type(.json_schema),
     "Input .tools must be NULL, a TOOL object, or a list of TOOL objects" = is.null(.tools) || S7_inherits(.tools, TOOL) || (is.list(.tools) && all(purrr::map_lgl(.tools, ~ S7_inherits(.x, TOOL)))),
     "Input .tool_choice must be NULL or a character (one of 'none', 'auto', 'required')" = is.null(.tool_choice) || (is.character(.tool_choice) && .tool_choice %in% c("none", "auto", "required")),
     "Input .dry_run must be logical" = is.logical(.dry_run) && length(.dry_run) == 1,
@@ -94,45 +199,57 @@ mistral_chat <- function(.llm,
   ) |>
     validate_inputs()
   
-  api_obj <- api_mistral(short_name = "mistral",
-                        long_name  = "Mistral",
-                        api_key_env_var = "MISTRAL_API_KEY")
+  # Create API object
+  api_obj <- api_mistral(
+    short_name = "mistral",
+    long_name = "Mistral",
+    api_key_env_var = "MISTRAL_API_KEY"
+  )
   
-  api_key <- get_api_key(api_obj,.dry_run)
+  # Get API key
+  api_key <- get_api_key(api_obj, .dry_run)
   
-  messages <- to_api_format(llm=.llm,
-                            api=api_obj,
-                            no_system=TRUE)
+  # Use the prepare_openai_request helper to set up common parameters
+  request_data <- prepare_mistral_request(
+    .llm = .llm,
+    .api = api_obj,
+    .model = .model,
+    .max_tokens = .max_tokens,
+    .frequency_penalty = .frequency_penalty,
+    .logit_bias = .logit_bias,
+    .presence_penalty = .presence_penalty,
+    .seed = .seed,
+    .stop = .stop,
+    .safe_prompt = .safe_prompt,
+    .min_tokens = .min_tokens,
+    .temperature = .temperature,
+    .top_p = .top_p,
+    .json_schema = .json_schema
+  )
   
-  #Put a single tool into a list if only one is provided. 
+  # Get components from the request data
+  request_body <- request_data$request_body
+  json <- request_data$json
+  
+  # Add streaming if requested
+  if (.stream) {
+    request_body$stream <- TRUE
+  }
+  
+  # Handle tools
   tools_def <- if (!is.null(.tools)) {
-    if (S7_inherits(.tools, TOOL))  list(.tools) else .tools
+    if (S7_inherits(.tools, TOOL)) list(.tools) else .tools
   } else {
     NULL
   }
   
-  #set options
-  mistral_request_body <- list(
-    model    = .model,
-    messages = messages,
-    temperature = .temperature,
-    random_seed = .seed,
-    max_tokens  = .max_tokens,
-    min_tokens  = .min_tokens,
-    stream      = .stream,
-    top_p        = .top_p,          
-    stop         = .stop,           
-    safe_prompt  = .safe_prompt,
-    tools = if(!is.null(tools_def)) tools_to_api(api_obj,tools_def) else NULL,
-    tool_choice = .tool_choice
-  ) |>
-    purrr::compact()
-  
-  # Add format to request body if applicable
-  if (.json == TRUE) {
-    mistral_request_body$response_format <- list(type="json_object")
+  # Add tools to request body if provided
+  if (!is.null(tools_def)) {
+    request_body$tools <- tools_to_api(api_obj, tools_def)
+    request_body$tool_choice <- .tool_choice
   }
   
+
   # Build the request
   request <- httr2::request("https://api.mistral.ai") |>
     httr2::req_url_path("/v1/chat/completions") |>
@@ -141,41 +258,48 @@ mistral_chat <- function(.llm,
       `Content-Type` = "application/json",
       .redact = "Authorization"
     ) |>
-    httr2::req_body_json(mistral_request_body)
+    httr2::req_body_json(request_body)
   
-  # Return only the request object in a dry run.
+  # Return only the request object in a dry run
   if (.dry_run) {
     return(request)  
   }
   
-  response <- perform_chat_request(request,api_obj,.stream,.timeout,.max_tries)
-  if(!is.null(response$raw$content$choices[[1]]$message$tool_calls)){
+  # Perform the request
+  response <- perform_chat_request(request, api_obj, .stream, .timeout, .max_tries)
+  
+  # Handle tool calls if any
+  if (!is.null(response$raw$content$choices[[1]]$message$tool_calls)) {
     tool_messages <- run_tool_calls(api_obj,
                                     response$raw$content$choices[[1]]$message$tool_calls,
                                     tools_def)
-    ##Append the tool call to API
-    mistral_request_body$messages <- mistral_request_body$messages |> 
-      append(tool_messages)
     
-    request <- request |>
-      httr2::req_body_json(data = mistral_request_body)
+    # Append the tool call to API
+    request_body$messages <- request_body$messages |> append(tool_messages)
     
-    response <- perform_chat_request(request,api_obj,.stream,.timeout,.max_tries)
+    # Update the request and perform it again
+    request <- request |> httr2::req_body_json(data = request_body)
+    response <- perform_chat_request(request, api_obj, .stream, .timeout, .max_tries)
   }
   
-  # Extract response headers for rate-limiting
-  track_rate_limit(api_obj,response$headers,.verbose)
-
+  # Extract assistant reply
+  assistant_reply <- response$assistant_reply
+  
+  # Track rate limit
+  track_rate_limit(api_obj, response$headers, .verbose)
+  
   # Add model's message to the history of the LLMMessage object
-  add_message(llm= .llm,
-              role = "assistant", 
-              content =  response$assistant_reply, 
-              json    = .json,
-              meta    = response$meta)  
+  add_message(
+    .llm = .llm,
+    .role = "assistant", 
+    .content = assistant_reply,
+    .json = json,
+    .meta = response$meta,
+    .logprobs = NULL
+  )
 }
 
 
-
 #' Generate Embeddings Using Mistral API
 #'
 #' @param .input A character vector of texts to embed or an `LLMMessage` object
@@ -245,23 +369,30 @@ mistral_embedding <- function(.input,
 
 #' Send a Batch of Requests to the Mistral API
 #'
+#' This function creates and submits a batch of messages to the Mistral API for asynchronous processing.
+#'
 #' @param .llms A list of LLMMessage objects containing conversation histories.
 #' @param .model The Mistral model version (default: "mistral-small-latest").
 #' @param .endpoint The API endpoint (default: "/v1/chat/completions").
 #' @param .metadata Optional metadata for the batch.
-#' @param .temperature Sampling temperature to use, between `0.0` and `1.5`. Higher values make the output more random (default: `0.7`).
+#' @param .temperature Sampling temperature to use, between `0.0` and `1.5` (default: `0.7`).
 #' @param .top_p Nucleus sampling parameter, between `0.0` and `1.0` (default: `1`).
 #' @param .max_tokens The maximum number of tokens to generate in the completion (default: `1024`).
 #' @param .min_tokens The minimum number of tokens to generate (optional).
+#' @param .frequency_penalty Numeric value (or NULL) for frequency penalty.
+#' @param .logit_bias A named list modifying the likelihood of specific tokens (or NULL).
+#' @param .presence_penalty Numeric value (or NULL) for presence penalty.
 #' @param .seed Random seed for deterministic outputs (optional).
-#' @param .stop Stop generation at specific tokens or strings (optional).
-#' @param .dry_run Logical; if `TRUE`, returns the prepared request without executing it (default: `FALSE`).
-#' @param .overwrite Logical; if `TRUE`, allows overwriting existing custom IDs (default: `FALSE`).
-#' @param .max_tries Maximum retry attempts for requests (default: `3`).
-#' @param .timeout Request timeout in seconds (default: `60`).
-#' @param .id_prefix Prefix for generating custom IDs (default: `"tidyllm_mistral_req_"`).
+#' @param .stop Sequence(s) at which to stop generation (optional).
+#' @param .safe_prompt Logical; if TRUE, injects a safety prompt (default: FALSE).
+#' @param .json_schema A JSON schema object for structured output (optional).
+#' @param .dry_run Logical; if TRUE, returns the prepared request without executing it (default: FALSE).
+#' @param .overwrite Logical; if TRUE, allows overwriting existing custom IDs (default: FALSE).
+#' @param .max_tries Maximum retry attempts for requests (default: 3).
+#' @param .timeout Request timeout in seconds (default: 60).
+#' @param .id_prefix Prefix for generating custom IDs (default: "tidyllm_mistral_req_").
 #'
-#' @return The `prepared_llms` list with the `batch_id` attribute attached.
+#' @return The prepared LLMMessage list with a batch_id attribute.
 #' @export
 send_mistral_batch <- function(.llms,
                                .model = "mistral-small-latest",
@@ -271,31 +402,42 @@ send_mistral_batch <- function(.llms,
                                .top_p = 1,
                                .max_tokens = 1024,
                                .min_tokens = NULL,
+                               .frequency_penalty = NULL,
+                               .logit_bias = NULL,
+                               .presence_penalty = NULL,
                                .seed = NULL,
                                .stop = NULL,
+                               .safe_prompt = FALSE,
+                               .json_schema = NULL,
                                .dry_run = FALSE,
                                .overwrite = FALSE,
                                .max_tries = 3,
                                .timeout = 60,
                                .id_prefix = "tidyllm_mistral_req_") {
-  # Validate inputs
+  
+  # Input validation
   c(
     ".llms must be a list of LLMMessage objects" = is.list(.llms) && all(sapply(.llms, S7_inherits, LLMMessage)),
     ".model must be a string" = is.character(.model),
     ".endpoint must be a string" = is.character(.endpoint),
     ".metadata must be NULL or a list" = is.null(.metadata) || is.list(.metadata),
-    ".dry_run must be logical" = is.logical(.dry_run),
-    ".overwrite must be logical" = is.logical(.overwrite),
-    ".id_prefix must be a string" = is.character(.id_prefix),
     ".temperature must be numeric between 0.0 and 1.5" = is.numeric(.temperature) && .temperature >= 0 && .temperature <= 1.5,
     ".top_p must be numeric between 0.0 and 1.0" = is.numeric(.top_p) && .top_p >= 0 && .top_p <= 1.0,
     ".max_tokens must be integer >= 0" = is_integer_valued(.max_tokens) && .max_tokens >= 0,
     ".min_tokens must be integer >= 0 if provided" = is.null(.min_tokens) || (is_integer_valued(.min_tokens) && .min_tokens >= 0),
+    ".frequency_penalty must be numeric or NULL" = is.null(.frequency_penalty) || is.numeric(.frequency_penalty),
+    ".logit_bias must be a list or NULL" = is.null(.logit_bias) || is.list(.logit_bias),
+    ".presence_penalty must be numeric or NULL" = is.null(.presence_penalty) || is.numeric(.presence_penalty),
     ".seed must be integer >= 0 if provided" = is.null(.seed) || (is_integer_valued(.seed) && .seed >= 0),
-    ".stop must be NULL or character" = is.null(.stop) || is.character(.stop)
+    ".stop must be NULL or character" = is.null(.stop) || is.character(.stop),
+    ".safe_prompt must be logical" = is.logical(.safe_prompt) && length(.safe_prompt) == 1,
+    ".json_schema must be NULL or a list or an ellmer type object" = is.null(.json_schema) || is.list(.json_schema) || is_ellmer_type(.json_schema),
+    ".dry_run must be logical" = is.logical(.dry_run),
+    ".overwrite must be logical" = is.logical(.overwrite),
+    ".id_prefix must be a string" = is.character(.id_prefix)
   ) |> validate_inputs()
   
-  # Initialize API object
+  # Initialize API object and get API key
   api_obj <- api_mistral(short_name = "mistral",
                          long_name = "Mistral",
                          api_key_env_var = "MISTRAL_API_KEY")
@@ -307,29 +449,34 @@ send_mistral_batch <- function(.llms,
                                           .id_prefix = .id_prefix, 
                                           .overwrite = .overwrite)
   
-  # Generate request lines for the batch
+  # Generate request lines for the batch using the common prepare function
   request_lines <- lapply(seq_along(prepared_llms), function(i) {
     custom_id <- names(prepared_llms)[i]
     
-    # Convert LLMMessage to API format
-    messages <- to_api_format(llm = prepared_llms[[i]],
-                              api = api_obj,
-                              no_system = TRUE)
+    # Prepare common request parameters using prepare_mistral_request
+    request_data <- prepare_mistral_request(
+      .llm = prepared_llms[[i]],
+      .api = api_obj,
+      .model = .model,
+      .min_tokens = .min_tokens,
+      .max_tokens = .max_tokens,
+      .frequency_penalty = .frequency_penalty,
+      .logit_bias = .logit_bias,
+      .presence_penalty = .presence_penalty,
+      .seed = .seed,
+      .stop = .stop,
+      .safe_prompt = .safe_prompt,
+      .temperature = .temperature,
+      .top_p = .top_p,
+      .json_schema = .json_schema
+    )
     
-    # Build the body for each request
-    body <- list(
-      model = .model,
-      messages = messages,
-      temperature = .temperature,
-      top_p = .top_p,
-      max_tokens = .max_tokens,
-      min_tokens = .min_tokens,
-      random_seed = .seed,
-      stop = .stop
-    ) |> purrr::compact()
-    
-    list(custom_id = custom_id, body = body) |> 
-      jsonlite::toJSON(auto_unbox = TRUE)
+    list(
+      custom_id = custom_id,
+      method = "POST",
+      url = .endpoint,
+      body = request_data$request_body
+    ) |> jsonlite::toJSON(auto_unbox = TRUE)
   })
   
   # Save the batch requests to a temporary .jsonl file
@@ -337,7 +484,7 @@ send_mistral_batch <- function(.llms,
   writeLines(unlist(request_lines), con = temp_file)
   
   if (.dry_run) {
-    return(temp_file)
+    return(readLines(temp_file))
   }
   
   # Upload the batch file
@@ -359,25 +506,26 @@ send_mistral_batch <- function(.llms,
   )
   
   batch_request <- httr2::request("https://api.mistral.ai/v1/batch/jobs") |>
-    httr2::req_headers(Authorization = sprintf("Bearer %s", api_key), `Content-Type` = "application/json") |>
+    httr2::req_headers(
+      Authorization = sprintf("Bearer %s", api_key),
+      `Content-Type` = "application/json"
+    ) |>
     httr2::req_body_json(batch_request_body)
   
   batch_response <- batch_request |>
     perform_generic_request(.timeout = .timeout, .max_tries = .max_tries)
   
-  if(batch_response$status==403){
-    paste0("Mistral Batch API error: ", batch_response$content$detail) |>
-      rlang::abort()
+  if(batch_response$status == 403) {
+    rlang::abort(sprintf("Mistral Batch API error: %s", batch_response$content$detail))
   }
   
-  # Attach batch_id as an attribute to .llms
+  # Attach batch_id as an attribute to the prepared LLMs
   batch_id <- batch_response$content$id
   attr(prepared_llms, "batch_id") <- batch_id
-
-  # Optionally, remove the temporary file
+  
+  # Optionally remove the temporary file
   unlink(temp_file)
   
-  # Return the prepared LLMs with attributes
   return(prepared_llms)
 }
 
@@ -571,6 +719,9 @@ fetch_mistral_batch <- function(.llms,
     ".timeout must be integer-valued numeric" = is_integer_valued(.timeout)
   ) |> validate_inputs()
   
+  .json <- attr(.llms, "json")
+  if (is.null(.json)) {.json <- FALSE}
+  
   # Preserve original names
   original_names <- names(.llms)
   
@@ -629,11 +780,11 @@ fetch_mistral_batch <- function(.llms,
     if (!is.null(result) && is.null(result$error) && result$response$status_code == 200) {
       assistant_reply <- result$response$body$choices$message$content
       meta_data <- extract_metadata(api_obj,result$response$body)
-      llm <- add_message(llm = .llms[[custom_id]],
-                         role = "assistant", 
-                         content =  assistant_reply,
-                         json = FALSE,
-                         meta = meta_data)
+      llm <- add_message(.llm = .llms[[custom_id]],
+                         .role = "assistant", 
+                         .content =  assistant_reply,
+                         .json = .json,
+                         .meta = meta_data)
       return(llm)
     } else {
       warning(sprintf("Result for custom_id %s was unsuccessful or not found", custom_id))
@@ -646,7 +797,8 @@ fetch_mistral_batch <- function(.llms,
   
   # Remove batch_id attribute before returning to avoid reuse conflicts
   attr(updated_llms, "batch_id") <- NULL
-
+  attr(updated_llms, "json") <- NULL
+  
   return(updated_llms)
 }
 
